# -*- coding: utf-8 -*-
"""
Helper functions for handling JSON data from Ostriz.
"""

from __future__ import absolute_import, unicode_literals

import datetime
import io
import json
import os

from collections import OrderedDict

import requests
import six

from dump2polarion import exporter
from dump2polarion.exceptions import Dump2PolarionException, NothingToDoException


def _get_json(location):
    """Reads JSON data from file or URL."""
    location = os.path.expanduser(location)
    try:
        if os.path.isfile(location):
            with io.open(location, encoding='utf-8') as json_data:
                return json.load(json_data, object_pairs_hook=OrderedDict).get('tests')
        elif 'http' in location:
            json_data = requests.get(location)
            if not json_data:
                raise Dump2PolarionException("Failed to download")
            return json.loads(json_data.text, object_pairs_hook=OrderedDict).get('tests')
        else:
            raise Dump2PolarionException("Invalid location")
    except Exception as err:
        raise Dump2PolarionException(
            "Failed to parse JSON from {}: {}".format(location, err))


def _get_testrun_id(version):
    """Gets testrun id out of the appliance_version file."""
    try:
        build_base = version.strip().split('-')[0].split('_')[0].replace('.', '_')
        zval = int(build_base.split('_')[3])
    except Exception:
        # not in expected format
        raise Dump2PolarionException("Cannot find testrun id")
    if zval < 10:
        pad_build = build_base[-1].zfill(2)
        return build_base[:-1] + pad_build
    return build_base


def _calculate_duration(start_time, finish_time):
    """Calculates how long it took to execute the testcase."""
    if not(start_time and finish_time):
        return 0
    start = datetime.datetime.fromtimestamp(start_time)
    finish = datetime.datetime.fromtimestamp(finish_time)
    duration = finish - start

    microseconds = float(('0.' + str(duration.microseconds)))
    return duration.seconds + microseconds


# pylint: disable=inconsistent-return-statements
def _get_testname(test_path):
    """Gets test name out of full test path."""
    path_end = test_path.find('.py/')
    if path_end:
        return test_path[path_end+4:]


def _parse_ostriz(ostriz_data):
    """Reads the content of the input JSON and returns testcases results."""
    if not ostriz_data:
        raise NothingToDoException("No data to import")

    results = []
    found_build = None
    for test_path, test_data in six.iteritems(ostriz_data):
        # make sure we are collecting data for the same build
        if found_build:
            if found_build != test_data.get('build'):
                continue
        # Every record should have "build" key. Skip if doesn't and
        # set `found_build` from first record where it's present.
        else:
            found_build = test_data.get('build')
            if not found_build:
                continue

        statuses = test_data.get('statuses')
        if not statuses:
            continue

        jenkins_data = test_data.get('jenkins', {})

        data = [
            ('title', test_data.get('test_name') or _get_testname(test_path)),
            ('verdict', statuses.get('overall')),
            ('source', test_data.get('source')),
            ('job_name', jenkins_data.get('job_name')),
            ('run', jenkins_data.get('build_number')),
            ('time', _calculate_duration(
                test_data.get('start_time'), test_data.get('finish_time')) or 0)
        ]
        test_id = test_data.get('polarion')
        if test_id:
            if isinstance(test_id, list):
                test_id = test_id[0]
            data.append(('test_id', test_id))

        results.append(OrderedDict(data))

    testrun_id = _get_testrun_id(found_build)
    return exporter.ImportedData(results=results, testrun=testrun_id)


# pylint: disable=unused-argument
def import_ostriz(location, **kwargs):
    """Reads Ostriz's data and returns imported data."""
    ostriz_data = _get_json(location)
    return _parse_ostriz(ostriz_data)



# encoding: utf-8
# pylint: disable=missing-docstring,redefined-outer-name,no-self-use,too-few-public-methods

from __future__ import unicode_literals

import io
import os
import logging

import pytest

from tests import conf


GENERIC_CONF = {
    'xunit_target': 'https://polarion.example.com/import/xunit',
    'testcase_taget': 'https://polarion.example.com/import/testcase',
    'xunit_queue': 'https://polarion.example.com/import/xunit-queue',
    'testcase_queue': 'https://polarion.example.com/import/testcase-queue'
}

RHCF3_PROPS = {
    'polarion-dry-run': False,
    'polarion-project-id': 'RHCF3',
    'polarion-testrun-status-id': 'inprogress',
    'polarion-response-test': 'test'
}

RHCF3_CONF = GENERIC_CONF.copy()
RHCF3_CONF['xunit_import_properties'] = RHCF3_PROPS

CMP_CONF = GENERIC_CONF.copy()
CMP_CONF['xunit_import_properties'] = RHCF3_PROPS.copy()
CMP_CONF['xunit_import_properties']['polarion-project-id'] = 'CMP'


@pytest.fixture(scope='module')
def config_prop():
    return RHCF3_CONF


@pytest.fixture(scope='module')
def config_prop_cmp():
    return CMP_CONF


@pytest.fixture(scope='function')
def config_e2e():
    conf_file = os.path.join(conf.DATA_PATH, 'dump2polarion.yaml')
    return str(conf_file)


class SimpleFormatter(object):
    def format(self, record):
        message = record.getMessage()
        if isinstance(message, bytes):
            message = message.decode('utf-8')
        return message


@pytest.yield_fixture
def captured_log():
    buff = io.StringIO()
    handler = logging.StreamHandler(buff)
    handler.setFormatter(SimpleFormatter())

    logger = logging.getLogger()
    logger.setLevel(logging.DEBUG)
    logger.addHandler(handler)

    yield buff

    logger.handlers.remove(handler)


@pytest.fixture(autouse=True)
def no_user_conf(monkeypatch):
    monkeypatch.setattr('dump2polarion.configuration.DEFAULT_USER_CONF', '')



# encoding: utf-8
# pylint: disable=missing-docstring,redefined-outer-name,no-self-use,protected-access

from __future__ import unicode_literals

import os
import copy
import io

from xml.etree import ElementTree

import pytest
from tests import conf

from dump2polarion.exceptions import Dump2PolarionException, NothingToDoException
from dump2polarion.exporter import ImportedData, XunitExport
from dump2polarion.importer import do_import
from dump2polarion.utils import get_unicode_str


@pytest.fixture(scope="module")
def records_ids():
    csv_file = os.path.join(conf.DATA_PATH, 'workitems_ids.csv')
    return do_import(csv_file)


@pytest.fixture(scope="module")
def records_names():
    csv_file = os.path.join(conf.DATA_PATH, 'workitems_ids.csv')
    records = do_import(csv_file)
    for res in records.results:
        res.pop('id')
    return records


def test_top_element(config_prop, records_ids):
    exporter = XunitExport('5_8_0_17', records_ids, config_prop, transform_func=lambda: None)
    top_element = exporter._top_element()
    parsed = '<testsuites><!--Generated for testrun 5_8_0_17--></testsuites>'.strip()
    top_element_str = get_unicode_str(ElementTree.tostring(top_element, 'utf-8').strip())
    assert top_element_str == parsed


class TestProperties(object):
    def test_properties_element(self, config_prop, records_ids):
        exporter = XunitExport(
            '5_8_0_17', records_ids, config_prop, transform_func=lambda: None)
        top_element = exporter._top_element()
        properties_element = exporter._properties_element(top_element)
        parsed = ('<properties>'
                  '<property name="polarion-testrun-id" value="5_8_0_17" />'
                  '<property name="polarion-dry-run" value="False" />'
                  '<property name="polarion-project-id" value="RHCF3" />'
                  '<property name="polarion-response-test" value="test" />'
                  '<property name="polarion-testrun-status-id" value="inprogress" />'
                  '</properties>'.strip())
        properties_str = get_unicode_str(ElementTree.tostring(properties_element, 'utf-8').strip())
        assert properties_str == parsed

    def test_properties_lookup_config(self, config_prop, records_names):
        new_config = copy.deepcopy(config_prop)
        new_config['xunit_import_properties']['polarion-lookup-method'] = "id"
        exporter = XunitExport(
            '5_8_0_17', records_names, new_config, transform_func=lambda: None)
        top_element = exporter._top_element()
        properties_element = exporter._properties_element(top_element)
        exporter._fill_lookup_prop(properties_element)
        properties_str = get_unicode_str(ElementTree.tostring(properties_element, 'utf-8').strip())
        assert '<property name="polarion-lookup-method" value="id" />' in properties_str

    def test_properties_invalid_lookup(self, config_prop, records_ids):
        new_config = copy.deepcopy(config_prop)
        new_config['xunit_import_properties']['polarion-lookup-method'] = 'invalid'
        exporter = XunitExport('5_8_0_17', records_ids, new_config)
        with pytest.raises(Dump2PolarionException) as excinfo:
            exporter.export()
        assert "Invalid value 'invalid' for the 'polarion-lookup-method'" in str(excinfo.value)


class TestE2E(object):
    def test_e2e_noresults(self, config_prop, records_ids):
        exporter = XunitExport(
            '5_8_0_17', records_ids, config_prop, transform_func=lambda arg: None)
        with pytest.raises(NothingToDoException) as excinfo:
            exporter.export()
        assert 'Nothing to export' in str(excinfo.value)

    def test_e2e_missing_results(self, config_prop):
        new_records = ImportedData(results=[], testrun=None)
        exporter = XunitExport(
            '5_8_0_17', new_records, config_prop, transform_func=lambda arg: None)
        with pytest.raises(NothingToDoException) as excinfo:
            exporter._fill_tests_results(None)
        assert 'Nothing to export' in str(excinfo.value)

    def test_e2e_ids_notransform(self, config_prop, records_ids):
        exporter = XunitExport(
            '5_8_0_17', records_ids, config_prop, transform_func=lambda arg: arg)
        complete = exporter.export()
        fname = 'complete_notransform.xml'
        with io.open(os.path.join(conf.DATA_PATH, fname), encoding='utf-8') as input_xml:
            parsed = input_xml.read()
        assert complete == parsed

    def test_e2e_ids_transform(self, config_prop, records_ids):
        exporter = XunitExport('5_8_0_17', records_ids, config_prop)
        complete = exporter.export()
        fname = 'complete_transform.xml'
        with io.open(os.path.join(conf.DATA_PATH, fname), encoding='utf-8') as input_xml:
            parsed = input_xml.read()
        assert complete == parsed

    def test_e2e_names_notransform(self, config_prop, records_names):
        exporter = XunitExport(
            '5_8_0_17', records_names, config_prop, transform_func=lambda arg: arg)
        complete = exporter.export()
        fname = 'complete_notransform_name.xml'
        with io.open(os.path.join(conf.DATA_PATH, fname), encoding='utf-8') as input_xml:
            parsed = input_xml.read()
        assert complete == parsed

    def test_e2e_names_transform(self, config_prop, records_names):
        exporter = XunitExport('5_8_0_17', records_names, config_prop)
        complete = exporter.export()
        fname = 'complete_transform_name.xml'
        with io.open(os.path.join(conf.DATA_PATH, fname), encoding='utf-8') as input_xml:
            parsed = input_xml.read()
        assert complete == parsed



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Polarion dumper for CFME Jenkins
"""

from __future__ import unicode_literals, absolute_import, print_function

import sys

from dump2polarion.dumper_cli import main


def get_testrun_id(version_file):
    """Gets testrun id out of the appliance_version file."""
    with open(version_file) as input_file:
        version = input_file.read().strip()
        version = version.split('-')[0].split('_')[0]
        build_base = version.replace('.', '_')
    try:
        zval = int(build_base.split('_')[3])
    except (IndexError, ValueError):
        # not in expected format
        return
    if zval < 10:
        pad_build = build_base[-1].zfill(2)
        return build_base[:-1] + pad_build
    return build_base


def tweak_args():
    """Tweaks args for polarion dumper."""
    new_argv = sys.argv[1:]

    for index, arg in enumerate(new_argv):
        if arg in ('-t', '--testrun-id'):
            break
    else:
        index = None

    testrun_id = get_testrun_id(new_argv[index + 1])
    if not testrun_id:
        print("Cannot find testrun id.", file=sys.stderr)
        sys.exit(1)

    # replace `-t appliance_version` with `-t testrun_id`
    new_argv[index + 1] = testrun_id

    return new_argv


if __name__ == '__main__':
    sys.exit(main(tweak_args()))



# encoding: utf-8
# pylint: disable=missing-docstring,redefined-outer-name,no-self-use,protected-access

from __future__ import unicode_literals

import os
import io
import datetime
import shutil

import pytest
from tests import conf

from dump2polarion.exporter import XunitExport
from dump2polarion.exceptions import Dump2PolarionException
from dump2polarion import dbtools


@pytest.fixture(scope='module')
def records_db():
    db_file = os.path.join(conf.DATA_PATH, 'workitems_ids.sqlite3')
    return dbtools.import_sqlite(db_file)


class TestDB(object):
    def test_testrun_id_exported(self):
        db_file = os.path.join(conf.DATA_PATH, 'workitems_ids.sqlite3')
        conn = dbtools._open_sqlite(db_file)
        testrun_id = dbtools._get_testrun_from_sqlite(conn)
        assert testrun_id == '5_8_0_17'

    def test_testrun_invalid_db(self):
        db_file = os.path.join(conf.DATA_PATH, 'ostriz.json')
        conn = dbtools._open_sqlite(db_file)
        testrun_id = dbtools._get_testrun_from_sqlite(conn)
        assert testrun_id is None

    def test_import_orig_data(self, records_db):
        assert hasattr(records_db, 'results')
        assert len(records_db.results) == 15
        assert 'id' in records_db.results[0]
        assert hasattr(records_db, 'testrun')
        assert records_db.testrun == '5_8_0_17'

    def test_import_time(self):
        db_file = os.path.join(conf.DATA_PATH, 'workitems_ids.sqlite3')
        older_than = datetime.datetime(2017, 6, 15)
        records = dbtools.import_sqlite(db_file, older_than=older_than)
        assert hasattr(records, 'results')
        assert len(records.results) == 14

    def test_open_nonexistent(self):
        db_file = 'nonexistent'
        with pytest.raises(Dump2PolarionException) as excinfo:
            dbtools._open_sqlite(db_file)
        assert 'No such file or directory' in str(excinfo.value)

    def test_exported_older_than(self, tmpdir):
        orig_db_file = os.path.join(conf.DATA_PATH, 'workitems_ids.sqlite3')
        older_than = datetime.datetime(2017, 6, 15)
        db_file = os.path.join(str(tmpdir), 'workitems_copy.sqlite3')
        shutil.copy(orig_db_file, db_file)
        dbtools.mark_exported_sqlite(db_file, older_than=older_than)
        conn = dbtools._open_sqlite(db_file)
        cur = conn.cursor()
        select = "SELECT count(*) FROM testcases WHERE exported == 'yes'"
        cur.execute(select)
        num = cur.fetchone()
        conn.close()
        assert num[0] == 12

    def test_exported_all(self, tmpdir):
        orig_db_file = os.path.join(conf.DATA_PATH, 'workitems_ids.sqlite3')
        db_file = os.path.join(str(tmpdir), 'workitems_copy.sqlite3')
        shutil.copy(orig_db_file, db_file)
        dbtools.mark_exported_sqlite(db_file)
        conn = dbtools._open_sqlite(db_file)
        cur = conn.cursor()
        select = "SELECT count(*) FROM testcases WHERE exported == 'yes'"
        cur.execute(select)
        num = cur.fetchone()
        conn.close()
        assert num[0] == 13

    def test_e2e_ids_notransform(self, config_prop, records_db):
        exporter = XunitExport(
            '5_8_0_17', records_db, config_prop, transform_func=lambda arg: arg)
        complete = exporter.export()
        fname = 'complete_notransform.xml'
        with io.open(os.path.join(conf.DATA_PATH, fname), encoding='utf-8') as input_xml:
            parsed = input_xml.read()
        assert complete == parsed

    def test_e2e_ids_transform(self, config_prop, records_db):
        exporter = XunitExport('5_8_0_17', records_db, config_prop)
        complete = exporter.export()
        fname = 'complete_transform.xml'
        with io.open(os.path.join(conf.DATA_PATH, fname), encoding='utf-8') as input_xml:
            parsed = input_xml.read()
        assert complete == parsed



# encoding: utf-8
# pylint: disable=missing-docstring,redefined-outer-name,no-self-use

import os
import io

from mock import patch

from tests import conf

from dump2polarion import csv2sqlite_cli
from dump2polarion import dbtools
from dump2polarion.exporter import XunitExport


class TestCSV2sqliteCLI(object):
    def test_get_args(self):
        args = csv2sqlite_cli.get_args(['-i', 'foo', '-o', 'bar'])
        assert args.input_file == 'foo'
        assert args.output_file == 'bar'
        assert args.log_level is None

    def test_e2e_ok(self, config_prop, tmpdir):
        input_file = os.path.join(conf.DATA_PATH, 'workitems_ids.csv')
        db_file = os.path.join(str(tmpdir), 'workitems_copy.sqlite3')
        args = ['-i', input_file, '-o', db_file]
        with patch('dump2polarion.csv2sqlite_cli.utils.init_log'):
            retval = csv2sqlite_cli.main(args)
        assert retval == 0

        records = dbtools.import_sqlite(db_file)
        exporter = XunitExport('5_8_0_17', records, config_prop)
        complete = exporter.export()
        fname = 'complete_transform.xml'
        with io.open(os.path.join(conf.DATA_PATH, fname), encoding='utf-8') as input_xml:
            parsed = input_xml.read()
        assert complete == parsed

    def test_e2e_invalid_csv(self, captured_log):
        non_file = 'nonexistent.csv'
        args = ['-i', non_file, '-o', non_file]
        retval = csv2sqlite_cli.main(args)
        assert retval == 1
        assert 'No such file or directory' in captured_log.getvalue()

    def test_e2e_not_csv(self, captured_log):
        non_file = 'nonexistent.txt'
        args = ['-i', non_file, '-o', non_file]
        retval = csv2sqlite_cli.main(args)
        assert retval == 1
        assert 'is in CSV format' in captured_log.getvalue()



# encoding: utf-8
# pylint: disable=missing-docstring,redefined-outer-name,no-self-use,invalid-name

from __future__ import unicode_literals

import os

import pytest

from tests import conf

from dump2polarion import utils
from dump2polarion.exceptions import Dump2PolarionException


class TestUtils(object):
    def test_get_unicode_char(self):
        unicode_str = utils.get_unicode_str(u'®')
        assert unicode_str == u'®'

    def test_get_unicode_num(self):
        unicode_str = utils.get_unicode_str(42)
        assert unicode_str == u'42'

    def test_get_unicode_basestring(self):
        unicode_str = utils.get_unicode_str('@'.encode('ascii'))
        assert unicode_str == u'@'

    def test_fill_testrun_present(self):
        fname = 'complete_notransform.xml'
        fname = os.path.join(conf.DATA_PATH, fname)
        xml_root = utils.get_xml_root(fname)
        utils.xunit_fill_testrun_id(xml_root, 'not_used')
        filled = utils.etree_to_string(xml_root)
        assert 'name="polarion-testrun-id" value="5_8_0_17"' in filled

    def test_fill_testrun_invalid_xml(self):
        xml_root = utils.get_xml_root_from_str('<invalid/>')
        with pytest.raises(Dump2PolarionException) as excinfo:
            utils.xunit_fill_testrun_id(xml_root, '5_8_0_17')
        assert 'missing <testsuites>' in str(excinfo.value)

    def test_fill_testrun_no_properties(self):
        xml_root = utils.get_xml_root_from_str('<testsuites/>')
        with pytest.raises(Dump2PolarionException) as excinfo:
            utils.xunit_fill_testrun_id(xml_root, '5_8_0_17')
        assert 'Failed to find <properties> in the XML file' in str(excinfo.value)

    def test_fill_testrun_missing(self):
        fname = 'properties.xml'
        xml_root = utils.get_xml_root(os.path.join(conf.DATA_PATH, fname))
        utils.xunit_fill_testrun_id(xml_root, '5_8_0_17')
        filled = utils.etree_to_string(xml_root)
        assert 'name="polarion-testrun-id" value="5_8_0_17"' in filled

    @pytest.mark.parametrize(
        'fname',
        (
            'testcases.xml',
            'complete_transform.xml',
        ))
    def test_nofill_response(self, fname):
        xml_root = utils.get_xml_root(os.path.join(conf.DATA_PATH, fname))
        name, value = utils.fill_response_property(xml_root)
        assert name == 'test'
        assert value == 'test'

    @pytest.mark.parametrize(
        'fname',
        (
            'testcases_noresponse.xml',
            'testcases_noresponse2.xml',
        ))
    def test_fill_testcase_response(self, fname):
        xml_root = utils.get_xml_root(os.path.join(conf.DATA_PATH, fname))
        name, value = utils.fill_response_property(xml_root)
        filled = utils.etree_to_string(xml_root)
        assert name == 'dump2polarion'
        assert value
        assert '<response-property name="dump2polarion" value=' in filled

    def test_fill_testsuites_response(self):
        fname = 'complete_transform_noresponse.xml'
        xml_root = utils.get_xml_root(os.path.join(conf.DATA_PATH, fname))
        name, value = utils.fill_response_property(xml_root)
        filled = utils.etree_to_string(xml_root)
        assert name == 'dump2polarion'
        assert value
        assert '<property name="polarion-response-dump2polarion" value=' in filled

    def test_fill_custom_testcase_response(self):
        fname = 'testcases_noresponse.xml'
        xml_root = utils.get_xml_root(os.path.join(conf.DATA_PATH, fname))
        name, value = utils.fill_response_property(xml_root, 'test', 'test')
        filled = utils.etree_to_string(xml_root)
        assert name == 'test'
        assert value == 'test'
        assert '<response-property name="test" value="test"' in filled
        # make sure response properties are on top
        assert '<testcases project-id="RHCF3">\n  <response-properties>' in filled

    def test_fill_custom_testsuites_response(self):
        fname = 'complete_transform_noresponse.xml'
        xml_root = utils.get_xml_root(os.path.join(conf.DATA_PATH, fname))
        name, value = utils.fill_response_property(xml_root, 'test', 'test')
        filled = utils.etree_to_string(xml_root)
        assert name == 'test'
        assert value == 'test'
        assert '<property name="polarion-response-test" value="test"' in filled

    def test_fill_invalid_testsuites_response(self):
        xml_root = utils.get_xml_root_from_str('<invalid/>')
        with pytest.raises(Dump2PolarionException) as excinfo:
            utils.fill_response_property(xml_root, 'test', 'test')
        assert 'XML file is not in expected format' in str(excinfo.value)

    def test_write_xml_gen(self, tmpdir):
        dirname = str(tmpdir)
        utils.write_xml('<xml />', output_loc=dirname)
        assert 'output-' in os.listdir(dirname)[0]

    def test_write_xml_loc(self, tmpdir):
        dirname = str(tmpdir)
        utils.write_xml('<xml />', output_loc=os.path.join(dirname, 'output123.xml'))
        assert 'output123.xml' in os.listdir(dirname)[0]

    def test_write_xml_file(self, tmpdir):
        dirname = str(tmpdir)
        utils.write_xml('<xml />', filename=os.path.join(dirname, 'output123.xml'))
        assert 'output123.xml' in os.listdir(dirname)[0]

    def test_write_xml_no_data(self, tmpdir):
        dirname = str(tmpdir)
        with pytest.raises(Dump2PolarionException) as excinfo:
            utils.write_xml('', filename=os.path.join(dirname, 'output123.xml'))
        assert 'No data to write' in str(excinfo.value)

    def test_invalid_xml_root(self):
        with pytest.raises(Dump2PolarionException) as excinfo:
            utils.get_xml_root('NONEXISTENT.xml')
        assert 'Failed to parse XML file' in str(excinfo.value)

    def test_invalid_xml_str(self):
        with pytest.raises(Dump2PolarionException) as excinfo:
            utils.get_xml_root_from_str(None)
        assert 'Failed to parse XML file' in str(excinfo.value)



# -*- coding: utf-8 -*-
# pylint: disable=logging-format-interpolation
"""
Utils for dump2polarion.
"""

from __future__ import absolute_import, unicode_literals

import datetime
import io
import logging
import os
import random
import string

from xml.etree import ElementTree

from dump2polarion.exceptions import Dump2PolarionException


# pylint: disable=invalid-name
logger = logging.getLogger(__name__)

_VERSION_ID = '21'
_NOT_EXPECTED_FORMAT_MSG = 'XML file is not in expected format'


def get_unicode_str(obj):
    """Makes sure obj is a unicode string."""
    try:
        # Python 2.x
        if isinstance(obj, unicode):
            return obj
        if isinstance(obj, str):
            return obj.decode('utf-8', errors='ignore')
        return unicode(obj)
    except NameError:
        # Python 3.x
        if isinstance(obj, str):
            return obj
        if isinstance(obj, bytes):
            return obj.decode('utf-8', errors='ignore')
        return str(obj)


def init_log(log_level):
    """Initializes logging."""
    log_level = log_level or 'INFO'
    logging.basicConfig(
        format='%(name)s:%(levelname)s:%(message)s',
        level=getattr(logging, log_level.upper(), logging.INFO))


def write_xml(xml, output_loc=None, filename=None):
    """Outputs the XML content into a file.

    Args:
        xml: string with XML document
        output_loc: file or directory for saving the file
        filename: file name that will be used if output_loc is directory
            If it is needed and is not supplied, it will be generated
    """
    if not xml:
        raise Dump2PolarionException("No data to write.")
    filename = filename or 'output-{}-{:%Y%m%d%H%M%S}.xml'.format(
        ''.join(random.sample(string.ascii_lowercase, 5)), datetime.datetime.now())
    if output_loc:
        filename_fin = os.path.expanduser(output_loc)
        if os.path.isdir(filename_fin):
            filename_fin = os.path.join(filename_fin, filename)
    else:
        filename_fin = filename

    with io.open(filename_fin, 'w', encoding='utf-8') as xml_file:
        xml_file.write(get_unicode_str(xml))
    logger.info("Data written to '{}'".format(filename_fin))


def get_xml_root(xml_file):
    """Returns XML root."""
    try:
        xml_tree = ElementTree.parse(os.path.expanduser(xml_file))
        xml_root = xml_tree.getroot()
    # pylint: disable=broad-except
    except Exception as err:
        raise Dump2PolarionException("Failed to parse XML file '{}': {}".format(xml_file, err))
    return xml_root


def get_xml_root_from_str(xml):
    """Returns XML root from string."""
    try:
        xml_root = ElementTree.fromstring(xml.encode('utf-8'))
    # pylint: disable=broad-except
    except Exception as err:
        raise Dump2PolarionException("Failed to parse XML file: {}".format(err))
    return xml_root


def etree_to_string(xml_root):
    """Returns string representation of element tree."""
    return get_unicode_str(ElementTree.tostring(xml_root, encoding='utf-8'))


def xunit_fill_testrun_id(xml_root, testrun_id):
    """Adds the polarion-testrun-id property when it's missing."""
    if xml_root.tag == 'testcases':
        return
    if xml_root.tag != 'testsuites':
        raise Dump2PolarionException(
            "{} {}".format(_NOT_EXPECTED_FORMAT_MSG, "- missing <testsuites>"))
    properties = xml_root.find('properties')
    if properties is None:
        raise Dump2PolarionException("Failed to find <properties> in the XML file")
    for prop in properties:
        if prop.get('name') == 'polarion-testrun-id':
            break
    else:
        if not testrun_id:
            raise Dump2PolarionException(
                "Failed to submit results to Polarion - missing testrun id")
        ElementTree.SubElement(properties, 'property',
                               {'name': 'polarion-testrun-id', 'value': str(testrun_id)})


def generate_response_property(name=None, value=None):
    """Generates response property."""
    name = name or 'dump2polarion'
    value = value or ''.join(random.sample(string.ascii_lowercase, 9)) + _VERSION_ID
    return (name, value)


def fill_response_property(xml_root, name=None, value=None):
    """Returns response property and fills it if missing."""
    name, value = generate_response_property(name, value)
    response_property = None

    if xml_root.tag == 'testsuites':
        properties = xml_root.find('properties')
        for prop in properties:
            prop_name = prop.get('name', '')
            if 'polarion-response-' in prop_name:
                response_property = (prop_name[len('polarion-response-'):], str(prop.get('value')))
                break
        else:
            prop_name = 'polarion-response-{}'.format(name)
            ElementTree.SubElement(properties, 'property', {'name': prop_name, 'value': value})
            response_property = (name, value)
    elif xml_root.tag == 'testcases':
        properties = xml_root.find('response-properties')
        if properties is None:
            properties = ElementTree.Element('response-properties')
            # response properties needs to be on top!
            xml_root.insert(0, properties)
        for prop in properties:
            if prop.tag == 'response-property':
                prop_name = prop.get('name')
                prop_value = prop.get('value')
                if prop_name and prop_value:
                    response_property = (prop_name, str(prop_value))
                break
        else:
            ElementTree.SubElement(properties, 'response-property', {'name': name, 'value': value})
            response_property = (name, value)
    else:
        raise Dump2PolarionException(_NOT_EXPECTED_FORMAT_MSG)

    return response_property



# encoding: utf-8
# pylint: disable=missing-docstring,redefined-outer-name,no-self-use

from __future__ import unicode_literals

import os


DATA_PATH = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'data')



# -*- coding: utf-8 -*-
# pylint: disable=logging-format-interpolation
"""
Submit results to the Polarion XUnit/TestCase Importer.
"""

from __future__ import absolute_import, unicode_literals

import logging
import os

import requests

from dump2polarion import configuration, utils, verify
from dump2polarion.exceptions import Dump2PolarionException

# requests package backwards compatibility mess
# pylint: disable=import-error,ungrouped-imports
from requests.packages.urllib3.exceptions import InsecureRequestWarning as IRWrequests
# pylint: disable=no-member
requests.packages.urllib3.disable_warnings(IRWrequests)
try:
    import urllib3
    from urllib3.exceptions import InsecureRequestWarning as IRWurllib3
    urllib3.disable_warnings(IRWurllib3)
except ImportError:
    pass


# pylint: disable=invalid-name
logger = logging.getLogger(__name__)


def _get_xml_root(xml_root, xml_str, xml_file):
    if xml_root is not None:
        return xml_root
    if xml_str:
        return utils.get_xml_root_from_str(xml_str)
    if xml_file:
        return utils.get_xml_root(xml_file)
    raise Dump2PolarionException("Failed to submit results to Polarion - no data supplied")


def _get_submit_target(xml_root, config):
    if xml_root.tag == 'testcases':
        target = config.get('testcase_taget')
    elif xml_root.tag == 'testsuites':
        target = config.get('xunit_target')
    else:
        raise Dump2PolarionException(
            "Failed to submit results to Polarion - submit target not found")
    return target


def _get_queue_url(xml_root, config):
    if xml_root.tag == 'testcases':
        target = config.get('testcase_queue')
    elif xml_root.tag == 'testsuites':
        target = config.get('xunit_queue')
    else:
        return
    return target


def _get_credentials(config, **kwargs):
    login = kwargs.get('user') or config.get('username') or os.environ.get('POLARION_USERNAME')
    pwd = kwargs.get('password') or config.get('password') or os.environ.get('POLARION_PASSWORD')

    if not all([login, pwd]):
        raise Dump2PolarionException("Failed to submit results to Polarion - missing credentials")

    return (login, pwd)


def get_job_id(response):
    """Returns job ID of the import."""
    try:
        parsed = response.json()
        return parsed['files']['results.xml']['job-id']
    # pylint: disable=broad-except
    except Exception:
        return


def submit(xml_str=None, xml_file=None, xml_root=None, config=None, **kwargs):
    """Submits results to the Polarion Importer."""
    try:
        config = config or configuration.get_config()
        xml_root = _get_xml_root(xml_root, xml_str, xml_file)
        credentials = _get_credentials(config, **kwargs)
        submit_target = _get_submit_target(xml_root, config)
        utils.xunit_fill_testrun_id(xml_root, kwargs.get('testrun_id'))
        xml_input = utils.etree_to_string(xml_root)
    except Dump2PolarionException as err:
        logger.error(err)
        return

    logger.info("Submitting results to {}".format(submit_target))
    files = {'file': ('results.xml', xml_input)}
    try:
        response = requests.post(submit_target, files=files, auth=credentials, verify=False)
    # pylint: disable=broad-except
    except Exception as err:
        logger.error(err)
        response = None

    if response is None:
        logger.error("Failed to submit results to {}".format(submit_target))
    elif response:
        logger.info("Results received by the Importer (HTTP status {})".format(
            response.status_code))
        logger.info("Job ID: {}".format(get_job_id(response)))
    else:
        logger.error("HTTP status {}: failed to submit results to {}".format(
            response.status_code, submit_target))

    return response


def submit_and_verify(xml_str=None, xml_file=None, xml_root=None, config=None, **kwargs):
    """Submits results to the Polarion Importer and checks that it was imported."""
    try:
        config = config or configuration.get_config()
        xml_root = _get_xml_root(xml_root, xml_str, xml_file)
        credentials = _get_credentials(config, **kwargs)
        queue_url = _get_queue_url(xml_root, config)
    except Dump2PolarionException as err:
        logger.error(err)
        return

    verify_import = True
    job_id = None
    if not kwargs.get('no_verify'):
        verification_queue = verify.QueueSearch(
            user=credentials[0], password=credentials[1], queue_url=queue_url)
        verify_import = verification_queue.queue_init()

    response = submit(xml_root=xml_root, config=config, **kwargs)

    if not verify_import:
        # we wanted to verify the import but it didn't happen for some reason
        return False

    if response:
        job_id = get_job_id(response)
    if not kwargs.get('no_verify') and job_id:
        response = verification_queue.verify_submit(
            job_id, timeout=kwargs.get('verify_timeout'), log_file=kwargs.get('log_file'))

    return bool(response)



# -*- coding: utf-8 -*-
# pylint: disable=missing-docstring,no-self-use,too-few-public-methods,protected-access


from __future__ import unicode_literals

import os

from mock import patch
from tests import conf

from dump2polarion import utils, submit


class TestSubmit(object):
    def test_missing_input(self, config_prop, captured_log):
        with patch('requests.post'):
            submit.submit('', config=config_prop)
        assert 'no data supplied' in captured_log.getvalue()

    def test_missing_credentials(self, config_prop, captured_log):
        with patch('requests.post'):
            submit.submit('<foo/>', config=config_prop)
        assert 'missing credentials' in captured_log.getvalue()

    def test_missing_target(self, config_prop, captured_log):
        with patch('requests.post'):
            submit.submit('<foo/>', config=config_prop, user='john', password='123')
        assert 'submit target not found' in captured_log.getvalue()

    def test_missing_testrun_id(self, config_prop, captured_log):
        with patch('requests.post'):
            submit.submit(
                '<testsuites><properties></properties></testsuites>',
                config=config_prop,
                user='john',
                password='123')
        assert 'missing testrun id' in captured_log.getvalue()

    def test_fill_testrun_submit(self, config_prop):
        fname = 'properties.xml'
        with patch('requests.post'):
            submit.submit(
                xml_file=os.path.join(conf.DATA_PATH, fname),
                testrun_id='5_8_0_17',
                config=config_prop,
                user='john',
                password='123')

    def test_get_testcases_taget(self, config_prop):
        xml_root = utils.get_xml_root_from_str('<testcases/>')
        response = submit._get_submit_target(xml_root, config_prop)
        assert 'testcase' in response

    def test_get_testsuites_taget(self, config_prop):
        xml_root = utils.get_xml_root_from_str('<testsuites/>')
        response = submit._get_submit_target(xml_root, config_prop)
        assert 'xunit' in response

    def test_file_testsuites_failure(self, config_prop, captured_log):
        class Response(object):
            def __init__(self):
                self.status_code = 404

            def __len__(self):
                return 0

        input_file = os.path.join(conf.DATA_PATH, 'complete_transform.xml')
        with patch('requests.post', return_value=Response()):
            response = submit.submit(
                xml_file=input_file, config=config_prop, user='john', password='123')
        assert not response
        assert 'HTTP status 404: failed to submit results' in captured_log.getvalue()

    def test_file_testsuites_none(self, config_prop, captured_log):
        input_file = os.path.join(conf.DATA_PATH, 'complete_transform.xml')
        with patch('requests.post', return_value=None):
            response = submit.submit(
                xml_file=input_file, config=config_prop, user='john', password='123')
        assert not response
        assert 'Failed to submit results' in captured_log.getvalue()

    def test_file_testsuites_exception(self, config_prop, captured_log):
        input_file = os.path.join(conf.DATA_PATH, 'complete_transform.xml')
        with patch('requests.post', side_effect=KeyError('request failed')):
            response = submit.submit(
                xml_file=input_file, config=config_prop, user='john', password='123')
        assert not response
        logged_data = captured_log.getvalue()
        assert 'request failed' in logged_data
        assert 'Failed to submit results' in logged_data

    def test_file_testsuites_success(self, config_prop, captured_log):
        input_file = os.path.join(conf.DATA_PATH, 'complete_transform.xml')
        with patch('requests.post'):
            response = submit.submit(
                xml_file=input_file, config=config_prop, user='john', password='123')
        assert response
        logged_data = captured_log.getvalue()
        assert 'Results received' in logged_data
        assert 'Job ID' in logged_data

    def test_file_testcases_success(self, config_prop, captured_log):
        input_file = os.path.join(conf.DATA_PATH, 'testcases.xml')
        with patch('requests.post'):
            response = submit.submit(
                xml_file=input_file, config=config_prop, user='john', password='123')
        assert response
        logged_data = captured_log.getvalue()
        assert 'Results received' in logged_data
        assert 'Job ID' in logged_data


class TestSubmitAndVerify(object):
    def test_missing_input(self, config_prop, captured_log):
        with patch('requests.post'):
            submit.submit_and_verify('', config=config_prop)
        assert 'no data supplied' in captured_log.getvalue()

    def test_verify_none(self, config_prop, captured_log):
        input_file = os.path.join(conf.DATA_PATH, 'complete_transform.xml')
        with patch('requests.post'), \
                patch('dump2polarion.verify.QueueSearch') as mock:
            instance = mock.return_value
            instance.queue_init.return_value = None
            instance.verify_submit.return_value = True
            response = submit.submit_and_verify(
                xml_file=input_file, config=config_prop, user='john', password='123')
        assert not response
        assert 'Results received' in captured_log.getvalue()

    def test_no_verify(self, config_prop, captured_log):
        input_file = os.path.join(conf.DATA_PATH, 'complete_transform.xml')
        with patch('requests.post'):
            response = submit.submit_and_verify(
                xml_file=input_file,
                config=config_prop,
                user='john',
                password='123',
                no_verify=True)
        assert response
        assert 'Results received' in captured_log.getvalue()

    def test_verify_failed(self, config_prop, captured_log):
        input_file = os.path.join(conf.DATA_PATH, 'complete_transform.xml')
        with patch('requests.post'), \
                patch('dump2polarion.verify.QueueSearch') as mock:
            instance = mock.return_value
            instance.queue_init.return_value = True
            instance.verify_submit.return_value = False
            response = submit.submit_and_verify(
                xml_file=input_file, config=config_prop, user='john', password='123')
        assert not response
        assert 'Results received' in captured_log.getvalue()

    def test_verify_success(self, config_prop, captured_log):
        input_file = os.path.join(conf.DATA_PATH, 'complete_transform.xml')
        with patch('requests.post'), \
                patch('dump2polarion.verify.QueueSearch') as mock:
            instance = mock.return_value
            instance.queue_init.return_value = True
            instance.verify_submit.return_value = True
            response = submit.submit_and_verify(
                xml_file=input_file, config=config_prop, user='john', password='123')
        assert response
        assert 'Results received' in captured_log.getvalue()



# -*- coding: utf-8 -*-
# pylint: disable=missing-docstring,redefined-outer-name,no-self-use,protected-access

from __future__ import unicode_literals

import os

try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

import pytest
from tests import conf

from dump2polarion import junittools
from dump2polarion.exceptions import Dump2PolarionException


class TestJunitImport(object):
    def test_import_orig_data(self):
        junit_file = os.path.join(conf.DATA_PATH, 'junit-report.xml')
        data = junittools.import_junit(junit_file)
        assert hasattr(data, 'results')
        assert len(data.results) == 7
        passed_num = failed_num = skipped_num = 0
        for result in data.results:
            if result['verdict'] == 'passed':
                passed_num += 1
            elif result['verdict'] == 'failed':
                assert result['comment']
                failed_num += 1
            elif result['verdict'] == 'skipped':
                assert result['comment']
                skipped_num += 1
        assert passed_num == 2
        assert failed_num == 2
        assert skipped_num == 3
        assert hasattr(data, 'testrun')
        assert data.testrun is None

    def test_invalid_input(self):
        invalid_content = str('foo')
        input_file = StringIO(invalid_content)
        with pytest.raises(Dump2PolarionException) as excinfo:
            junittools.import_junit(input_file)
        input_file.close()
        assert 'Failed to parse XML file' in str(excinfo.value)



# encoding: utf-8
# pylint: disable=missing-docstring,redefined-outer-name,no-self-use,protected-access

from __future__ import unicode_literals

import pytest

from dump2polarion import configuration
from dump2polarion.exceptions import Dump2PolarionException


class TestConfiguration(object):
    def test_nonexistant(self):
        with pytest.raises(Dump2PolarionException) as excinfo:
            configuration.get_config('nonexistant')
        assert "Cannot open config file 'nonexistant'" in str(excinfo.value)

    def test_default(self):
        with pytest.raises(Dump2PolarionException) as excinfo:
            configuration.get_config()
        assert 'Failed to find following keys in config file' in str(excinfo.value)

    def test_user(self, config_e2e):
        cfg = configuration.get_config(config_e2e)
        assert cfg['xunit_import_properties']['polarion-dry-run'] is False
        assert cfg['username'] == 'user1'
        assert cfg['xunit_import_properties']['polarion-project-id'] == 'RHCF3'

    def test_check_config(self):
        cfg = {}
        with pytest.raises(Dump2PolarionException) as excinfo:
            configuration._check_config(cfg)
        assert 'Failed to find following keys in config file' in str(excinfo.value)



# -*- coding: utf-8 -*-
# pylint: disable=logging-format-interpolation
"""
Verifies that data were updated in Polarion.
"""

from __future__ import absolute_import, unicode_literals

import logging
import os
import time

import requests


# pylint: disable=invalid-name
logger = logging.getLogger(__name__)


class QueueSearch(object):
    """Search for job in the completed jobs queue."""

    _DEFAULT_TIMEOUT = 600
    _DEFAULT_DELAY = 10

    def __init__(self, user, password, queue_url):
        self.user = user
        self.password = password
        self.queue_url = queue_url
        self.last_id = None
        self.skip = False
        self._check_setup()

    def _check_setup(self):
        """Checks that all the data that is needed for submit verification is available."""
        if not self.queue_url:
            logger.error(
                'The queue url is not configured, skipping submit verification')
            self.skip = True
            return

        if not all([self.user, self.password]):
            logger.error('Missing credentials, skipping submit verification')
            self.skip = True
            return

    def download_queue(self, jobs_per_page=50, current_page=1):
        """Downloads data of completed jobs."""
        if self.skip:
            return

        url = '{0}?jobtype=completed&jobsPerPage={1}&currentPage={2}'.format(
            self.queue_url, jobs_per_page, current_page)
        try:
            response = requests.get(
                url,
                auth=(self.user, self.password),
                verify=False,
                headers={'Accept': 'application/json'}
            )
            if response:
                response = response.json()
            else:
                response = None
        # pylint: disable=broad-except
        except Exception as err:
            logger.error(err)
            response = None

        return response

    def find_job(self, job_id, last_id, max_depth=10, _current_page=1):
        """Finds the job in the completed job queue."""
        if self.skip:
            return

        json_data = self.download_queue(current_page=_current_page)
        if not json_data:
            return

        jobs = json_data['jobs']
        first_id = jobs[0]['id']
        if _current_page == 1:
            self.last_id = first_id
        for job in jobs:
            cur_id = job.get('id')
            if cur_id == job_id:
                return job
            elif cur_id == last_id:
                return
        if _current_page >= max_depth or _current_page >= json_data.get('maxPages', 0):
            return

        return self.find_job(job_id, last_id, _current_page=_current_page + 1)

    def wait_for_job(self, job_id, timeout=_DEFAULT_TIMEOUT, delay=_DEFAULT_DELAY):
        """Waits until the job appears in the completed job queue."""
        if self.skip:
            return

        logger.debug('Waiting up to {} sec for completion of the job ID {}'.format(timeout, job_id))

        countdown = timeout
        while countdown > 0:
            job = self.find_job(job_id, self.last_id)
            if job:
                return job
            time.sleep(delay)
            countdown -= delay

        logger.error(
            'Timed out while waiting for completion of the job ID {}. '
            'Results not updated.'.format(job_id))

    # pylint: disable=no-self-use
    def _check_outcome(self, job):
        """Parses returned message and checks submit outcome."""
        status = job.get('status') if job else None
        if not status:
            return False

        if status.lower() == 'success':
            logger.info('Results successfully updated!')
            return True
        logger.error('Status = {}, results not updated'.format(status))
        return False

    # pylint: disable=no-self-use
    def _download_log(self, url, output_file):
        """Saves log returned by the message bus."""
        logger.info("Saving log {} to {}".format(url, output_file))

        def _do_log_download():
            try:
                return requests.get(url)
            # pylint: disable=broad-except
            except Exception as err:
                logger.error(err)

        # log file may not be ready yet, wait a bit
        for __ in range(5):
            log_data = _do_log_download()
            if log_data or log_data is None:
                break
            time.sleep(2)

        if not log_data:
            logger.error("Failed to download log file '{}'.".format(url))
            return
        with open(os.path.expanduser(output_file), 'wb') as out:
            out.write(log_data.content)

    def get_log(self, job, log_file=None):
        """Get log or log url of the job."""
        if not job:
            return

        url = job.get('logstashURL')
        if url:
            if log_file:
                self._download_log(url, log_file)
            else:
                logger.info('Submit log: {}'.format(url))

    def queue_init(self):
        """Initializes the instance with the last job in the completed queue."""
        if self.skip:
            return

        json_data = self.download_queue(jobs_per_page=1)
        if json_data:
            self.last_id = json_data['jobs'][0]['id']
            return True

        logger.error('Failed to initialize.')
        self.skip = True
        return False

    def verify_submit(self, job_id, timeout=_DEFAULT_TIMEOUT, delay=_DEFAULT_DELAY, **kwargs):
        """Verifies that the results were successfully submitted."""
        job = self.wait_for_job(job_id, timeout, delay)
        self.get_log(job, log_file=kwargs.get('log_file'))

        return self._check_outcome(job)



# -*- coding: utf-8 -*-
# pylint: disable=missing-docstring,redefined-outer-name,no-self-use,protected-access,invalid-name

from __future__ import unicode_literals

import os

from mock import patch

from dump2polarion import verify


LAST_ID = {
    "jobs": [{
        "id": 17975
    }]
}

SEARCH_QUEUE = {
    "jobsPerPage": 50,
    "maxPages": 2,
    "currentPage": 1,
    "jobs": [{
        "id": 17977,
        "status": "SUCCESS",
        "logstashURL": "http://logstash/00017977.log"
    }, {
        "id": 17974,
        "status": "FAILED",
        "logstashURL": "http://logstash/00017974.log"
    }, {
        "id": 17976,
        "status": "SUCCESS",
        "logstashURL": "http://logstash/00017976.log"
    }]
}


class DummyResponse(object):
    def __init__(self, response):
        self.status_code = 200
        self.response = response

    def __len__(self):
        return 1

    def json(self):
        return self.response

    @property
    def content(self):
        return self.response


def download_queue_init_ok(**kwargs):
    return LAST_ID


def download_queue_data(**kwargs):
    return SEARCH_QUEUE


class TestQueueSearch(object):

    # object init
    def test_user_missing(self, captured_log):
        vq = verify.QueueSearch(None, 'foo', 'bar')
        assert 'Missing credentials, skipping' in captured_log.getvalue()
        assert vq.skip is True

    def test_password_missing(self, captured_log):
        vq = verify.QueueSearch(None, 'foo', 'bar')
        assert 'Missing credentials, skipping' in captured_log.getvalue()
        assert vq.skip is True

    def test_url_missing(self, captured_log):
        vq = verify.QueueSearch('foo', 'bar', None)
        assert 'The queue url is not configured, skipping' in captured_log.getvalue()
        assert vq.skip is True

    def test_init(self):
        vq = verify.QueueSearch('foo', 'bar', 'baz')
        assert vq.skip is False

    # queue init
    def test_queue_init_ok(self):
        vq = verify.QueueSearch('foo', 'bar', 'baz')
        vq.download_queue = download_queue_init_ok
        assert vq.queue_init()
        assert vq.skip is False
        assert vq.last_id == 17975

    def test_queue_init_failed(self, captured_log):
        vq = verify.QueueSearch('foo', 'bar', 'baz')
        vq.download_queue = lambda **kwargs: None
        assert vq.queue_init() is False
        assert vq.skip is True
        assert vq.last_id is None
        assert 'Failed to initialize' in captured_log.getvalue()

    # download queue
    def test_download_queue_none(self):
        with patch('requests.get', return_value=DummyResponse(None)):
            vq = verify.QueueSearch('foo', 'bar', 'baz')
            response = vq.download_queue()
        assert response is None

    def test_download_queue_data(self):
        with patch('requests.get', return_value=DummyResponse(SEARCH_QUEUE)):
            vq = verify.QueueSearch('foo', 'bar', 'baz')
            response = vq.download_queue()
        assert response == SEARCH_QUEUE

    def test_download_queue_failed(self):
        with patch('requests.get', return_value=False):
            vq = verify.QueueSearch('foo', 'bar', 'baz')
            response = vq.download_queue()
        assert response is None

    def test_download_queue_exception(self, captured_log):
        with patch('requests.get', side_effect=Exception('TestFail')):
            vq = verify.QueueSearch('foo', 'bar', 'baz')
            response = vq.download_queue()
        assert response is None
        assert 'TestFail' in captured_log.getvalue()

    # find job in queue
    def test_job_not_found(self):
        vq = verify.QueueSearch('foo', 'bar', 'baz')
        vq.download_queue = download_queue_data
        outcome = vq.find_job(17978, 17975)
        assert vq.last_id == 17977
        assert outcome is None

    def test_job_last_found(self):
        vq = verify.QueueSearch('foo', 'bar', 'baz')
        vq.download_queue = download_queue_data
        outcome = vq.find_job(17978, 17976)
        assert vq.last_id == 17977
        assert outcome is None

    def test_job_found(self):
        vq = verify.QueueSearch('foo', 'bar', 'baz')
        vq.download_queue = download_queue_data
        outcome = vq.find_job(17977, 17976)
        assert vq.last_id == 17977
        assert outcome is SEARCH_QUEUE['jobs'][0]

    # job log handling
    def test_get_log_failed(self, tmpdir, captured_log):
        log_file = os.path.join(str(tmpdir), 'out.log')
        job = {'logstashURL': 'foo'}
        with patch('requests.get', return_value=None):
            vq = verify.QueueSearch('foo', 'bar', 'baz')
            vq.get_log(job, log_file)
        assert not os.path.exists(log_file)
        assert 'Failed to download log file' in captured_log.getvalue()

    def test_get_log_saved(self, tmpdir):
        log_file = os.path.join(str(tmpdir), 'out.log')
        job = {'logstashURL': 'foo'}
        with patch('requests.get', return_value=DummyResponse(b'content')):
            vq = verify.QueueSearch('foo', 'bar', 'baz')
            vq.get_log(job, log_file)
        assert os.path.exists(log_file)

    def test_get_log_displayed(self, captured_log):
        job = {'logstashURL': 'foo'}
        with patch('requests.get', return_value=None):
            vq = verify.QueueSearch('foo', 'bar', 'baz')
            vq.get_log(job)
        assert 'Submit log: foo' in captured_log.getvalue()

    def test_get_log_exception(self, tmpdir, captured_log):
        log_file = os.path.join(str(tmpdir), 'out.log')
        job = {'logstashURL': 'foo'}
        with patch('requests.get', side_effect=Exception('TestFail')):
            vq = verify.QueueSearch('foo', 'bar', 'baz')
            vq.get_log(job, log_file)
        assert not os.path.exists(log_file)
        assert 'TestFail' in captured_log.getvalue()

    # verify submit
    def test_queue_empty_queue(self, captured_log):
        vq = verify.QueueSearch('foo', 'bar', 'baz')
        vq.download_queue = lambda **kwargs: None
        vq.last_id = 17975
        outcome = vq.verify_submit(17974, timeout=0.0000001, delay=0.0000001)
        assert outcome is False
        assert 'not updated' in captured_log.getvalue()

    def test_queue_submit_failed(self, captured_log):
        vq = verify.QueueSearch('foo', 'bar', 'baz')
        vq.download_queue = download_queue_data
        vq.last_id = 17975
        outcome = vq.verify_submit(17974, timeout=0.0000001, delay=0.0000001)
        assert outcome is False
        assert 'Status = FAILED' in captured_log.getvalue()

    def test_queue_submit_not_found(self, captured_log):
        vq = verify.QueueSearch('foo', 'bar', 'baz')
        vq.download_queue = download_queue_data
        vq.last_id = 17975
        outcome = vq.verify_submit(17978, timeout=0.0000001, delay=0.0000001)
        assert outcome is False
        assert 'not updated' in captured_log.getvalue()

    def test_queue_submit_found(self, captured_log):
        vq = verify.QueueSearch('foo', 'bar', 'baz')
        vq.download_queue = download_queue_data
        vq.last_id = 17975
        outcome = vq.verify_submit(17977, timeout=0.0000001, delay=0.0000001)
        assert outcome
        assert 'successfully updated' in captured_log.getvalue()

    def test_queue_submit_skip(self):
        vq = verify.QueueSearch('foo', 'bar', 'baz')
        vq.skip = True
        outcome = vq.verify_submit(17977, timeout=0.0000001, delay=0.0000001)
        assert outcome is False



# encoding: utf-8
# pylint: disable=missing-docstring,redefined-outer-name,no-self-use,protected-access

from __future__ import unicode_literals

import os
import io

import pytest

from mock import Mock, patch
from tests import conf

from dump2polarion import ostriztools
from dump2polarion.exceptions import Dump2PolarionException
from dump2polarion.exporter import XunitExport


@pytest.fixture(scope="module")
def records_json():
    json_file = os.path.join(conf.DATA_PATH, 'ostriz.json')
    return ostriztools.import_ostriz(json_file)


@pytest.fixture(scope="module")
def records_json_cmp():
    json_file = os.path.join(conf.DATA_PATH, 'ostriz_cmp.json')
    return ostriztools.import_ostriz(json_file)


@pytest.fixture(scope="module")
def records_json_search():
    json_file = os.path.join(conf.DATA_PATH, 'ostriz_search.json')
    return ostriztools.import_ostriz(json_file)


class TestOstriz(object):
    def test_testrun_id_simple(self):
        testrun_id = ostriztools._get_testrun_id('5.8.0.17')
        assert testrun_id == '5_8_0_17'

    def test_testrun_id_build(self):
        testrun_id = ostriztools._get_testrun_id('5.8.0.17-20170525183055_6317a22')
        assert testrun_id == '5_8_0_17'

    def test_testrun_id_fill(self):
        testrun_id = ostriztools._get_testrun_id('5.8.0.7-2017')
        assert testrun_id == '5_8_0_07'

    def test_testrun_id_invalid(self):
        with pytest.raises(Dump2PolarionException) as excinfo:
            ostriztools._get_testrun_id('INVALID')
        assert 'Cannot find testrun id' in str(excinfo.value)

    def test_duration_good(self):
        duration = ostriztools._calculate_duration(1495766591.151192, 1495768544.573208)
        assert duration == 1953.422016

    def test_duration_bad_finish(self):
        duration = ostriztools._calculate_duration(1495766591.151192, None)
        assert duration == 0

    def test_duration_bad_start(self):
        duration = ostriztools._calculate_duration(0, 1495768544.573208)
        assert duration == 0

    def test_import_orig_data(self, records_json):
        assert hasattr(records_json, 'results')
        assert len(records_json.results) == 6
        assert 'title' in records_json.results[0]
        assert hasattr(records_json, 'testrun')
        assert records_json.testrun == '5_8_0_17'

    def test_invalid_json(self):
        fname = 'junit-report.xml'
        with pytest.raises(Dump2PolarionException) as excinfo:
            ostriztools.import_ostriz(os.path.join(conf.DATA_PATH, fname))
        assert 'Failed to parse JSON' in str(excinfo.value)

    def test_remote_invalid_json(self):
        with patch('requests.get', return_value=False):
            with pytest.raises(Dump2PolarionException) as excinfo:
                ostriztools.import_ostriz('https://foo')
        assert 'Failed to parse JSON' in str(excinfo.value)

    def test_remote_json(self, records_json):
        json_file = os.path.join(conf.DATA_PATH, 'ostriz.json')
        with io.open(json_file, encoding='utf-8') as input_json:
            parsed = input_json.read()
        retval = Mock()
        retval.text = parsed
        with patch('requests.get', return_value=retval):
            loaded_json = ostriztools.import_ostriz('https://foo')
        assert loaded_json == records_json

    def test_no_json(self):
        with pytest.raises(Dump2PolarionException) as excinfo:
            ostriztools.import_ostriz('NONEXISTENT.json')
        assert 'Invalid location' in str(excinfo.value)

    def test_empty_json(self):
        with pytest.raises(Dump2PolarionException) as excinfo:
            ostriztools._parse_ostriz('')
        assert 'No data to import' in str(excinfo.value)

    def test_e2e_ids_notransform(self, config_prop, records_json):
        exporter = XunitExport(
            '5_8_0_17', records_json, config_prop, transform_func=lambda arg: arg)
        complete = exporter.export()
        fname = 'ostriz_notransform.xml'
        with io.open(os.path.join(conf.DATA_PATH, fname), encoding='utf-8') as input_xml:
            parsed = input_xml.read()
        assert complete == parsed

    def test_e2e_ids_transform(self, config_prop, records_json):
        exporter = XunitExport('5_8_0_17', records_json, config_prop)
        complete = exporter.export()
        fname = 'ostriz_transform.xml'
        with io.open(os.path.join(conf.DATA_PATH, fname), encoding='utf-8') as input_xml:
            parsed = input_xml.read()
        assert complete == parsed

    def test_e2e_cmp_ids_transform(self, config_prop_cmp, records_json_cmp):
        exporter = XunitExport('5_8_0_17', records_json_cmp, config_prop_cmp)
        complete = exporter.export()
        fname = 'ostriz_transform_cmp.xml'
        with io.open(os.path.join(conf.DATA_PATH, fname), encoding='utf-8') as input_xml:
            parsed = input_xml.read()
        assert complete == parsed

    def test_e2e_ids_search_transform(self, config_prop, records_json_search):
        exporter = XunitExport('5_8_0_17', records_json_search, config_prop)
        complete = exporter.export()
        fname = 'ostriz_search_transform.xml'
        with io.open(os.path.join(conf.DATA_PATH, fname), encoding='utf-8') as input_xml:
            parsed = input_xml.read()
        assert complete == parsed



# -*- coding: utf-8 -*-
# pylint: disable=logging-format-interpolation
"""
Dump testcases results to xunit file for submitting to the Polarion XUnit Importer.
"""

from __future__ import absolute_import, unicode_literals

import datetime

from collections import namedtuple

from xml.dom import minidom
from xml.etree import ElementTree

from dump2polarion import transform, utils
from dump2polarion.exceptions import Dump2PolarionException, NothingToDoException
from dump2polarion.verdicts import Verdicts


ImportedData = namedtuple('ImportedData', 'results testrun')


class XunitExport(object):
    """Exports testcases results into Polarion xunit."""
    def __init__(self, testrun_id, tests_records, config, transform_func=None):
        self.testrun_id = testrun_id
        self.tests_records = tests_records
        self.config = config
        self._lookup_prop = ''
        self._transform_func = transform_func or transform.get_results_transform(config)

    def _top_element(self):
        """Returns top XML element."""
        top = ElementTree.Element('testsuites')
        comment = ElementTree.Comment("Generated for testrun {}".format(self.testrun_id))
        top.append(comment)
        return top

    def _properties_element(self, parent_element):
        """Returns properties XML element."""
        testsuites_properties = ElementTree.SubElement(parent_element, 'properties')

        ElementTree.SubElement(testsuites_properties, 'property',
                               {'name': 'polarion-testrun-id', 'value': str(self.testrun_id)})

        for name, value in sorted(self.config['xunit_import_properties'].items()):
            if name == 'polarion-lookup-method':
                lookup_prop = str(value).lower()
                if lookup_prop not in ('id', 'name', 'custom'):
                    raise Dump2PolarionException(
                        "Invalid value '{}' for the 'polarion-lookup-method' property".format(
                            str(value)))
                self._lookup_prop = lookup_prop
            else:
                ElementTree.SubElement(testsuites_properties, 'property',
                                       {'name': name, 'value': str(value)})

        return testsuites_properties

    def _fill_lookup_prop(self, testsuites_properties):
        """Fills the polarion-lookup-method property."""
        if not self._lookup_prop:
            raise Dump2PolarionException(
                "Failed to set the 'polarion-lookup-method' property")

        ElementTree.SubElement(testsuites_properties, 'property',
                               {'name': 'polarion-lookup-method', 'value': self._lookup_prop})

    def _testsuite_element(self, parent_element):
        """Returns testsuite XML element."""
        testsuite = ElementTree.SubElement(
            parent_element,
            'testsuite',
            {'name': 'Import for {} - {} testrun'.format(
                self.config['xunit_import_properties']['polarion-project-id'], self.testrun_id)})
        return testsuite

    @staticmethod
    def _fill_verdict(verdict, result, testcase, records):
        # xunit Pass maps to Passed in Polarion
        if verdict in Verdicts.PASS:
            records['passed'] += 1
        # xunit Failure maps to Failed in Polarion
        elif verdict in Verdicts.FAIL:
            records['failures'] += 1
            verdict_data = {'type': 'failure'}
            if result.get('comment'):
                verdict_data['message'] = utils.get_unicode_str(result['comment'])
            ElementTree.SubElement(testcase, 'failure', verdict_data)
        # xunit Error maps to Blocked in Polarion
        elif verdict in Verdicts.SKIP:
            records['skipped'] += 1
            verdict_data = {'type': 'error'}
            if result.get('comment'):
                verdict_data['message'] = utils.get_unicode_str(result['comment'])
            ElementTree.SubElement(testcase, 'error', verdict_data)
        # xunit Skipped maps to Waiting in Polarion
        elif verdict in Verdicts.WAIT:
            records['waiting'] += 1
            verdict_data = {'type': 'skipped'}
            if result.get('comment'):
                verdict_data['message'] = utils.get_unicode_str(result['comment'])
            ElementTree.SubElement(testcase, 'skipped', verdict_data)

    def _gen_testcase(self, parent_element, result, records):
        """Creates XML element for given testcase result and update testcases records."""
        if self._transform_func:
            result = self._transform_func(result)
            if not result:
                return
        verdict = result.get('verdict')
        if not verdict:
            return
        verdict = verdict.strip().lower()
        if verdict not in Verdicts.PASS + Verdicts.FAIL + Verdicts.SKIP + Verdicts.WAIT:
            return
        testcase_id = result.get('id')
        testcase_title = result.get('title')

        if self._lookup_prop:
            if not testcase_id and self._lookup_prop != 'name':
                return
            if not testcase_title and self._lookup_prop == 'name':
                return
        else:
            if testcase_id:
                self._lookup_prop = 'id'
            elif testcase_title:
                self._lookup_prop = 'name'

        testcase_time = float(result.get('time') or result.get('duration') or 0)
        records['time'] += testcase_time

        testcase_data = {
            'name': testcase_title or testcase_id,
            'time': str(testcase_time)}
        if result.get('classname'):
            testcase_data['classname'] = result['classname']
        testcase = ElementTree.SubElement(parent_element, 'testcase', testcase_data)

        self._fill_verdict(verdict, result, testcase, records)

        if result.get('stdout'):
            system_out = ElementTree.SubElement(testcase, 'system-out')
            system_out.text = utils.get_unicode_str(result['stdout'])

        if result.get('stderr'):
            system_err = ElementTree.SubElement(testcase, 'system-err')
            system_err.text = utils.get_unicode_str(result['stderr'])

        properties = ElementTree.SubElement(testcase, 'properties')
        ElementTree.SubElement(
            properties,
            'property',
            {'name': 'polarion-testcase-id', 'value': testcase_id or testcase_title}
        )
        if verdict in Verdicts.PASS and result.get('comment'):
            ElementTree.SubElement(
                properties, 'property',
                {'name': 'polarion-testcase-comment',
                 'value': utils.get_unicode_str(result['comment'])}
            )

    def _fill_tests_results(self, testsuite_element):
        """Creates records for all testcases results."""
        if not self.tests_records.results:
            raise NothingToDoException("Nothing to export")

        records = dict(passed=0, skipped=0, failures=0, waiting=0, time=0.0)
        for testcase_result in self.tests_records.results:
            self._gen_testcase(testsuite_element, testcase_result, records)

        tests_num = (
            records['passed'] +
            records['skipped'] +
            records['failures'] +
            records['waiting'])

        if tests_num == 0:
            raise NothingToDoException("Nothing to export")

        testsuite_element.set('errors', str(records['skipped']))
        testsuite_element.set('failures', str(records['failures']))
        testsuite_element.set('skipped', str(records['waiting']))
        testsuite_element.set('time', '{0:.4f}'.format(records['time']))
        testsuite_element.set('tests', str(tests_num))

    @staticmethod
    def _prettify(top_element):
        """Returns a pretty-printed XML."""
        rough_string = ElementTree.tostring(top_element, 'utf-8')
        reparsed = minidom.parseString(rough_string)
        return utils.get_unicode_str(reparsed.toprettyxml(indent='  ', encoding='utf-8'))

    def export(self):
        """Returns xunit XML."""
        top = self._top_element()
        properties = self._properties_element(top)
        testsuite = self._testsuite_element(top)
        self._fill_tests_results(testsuite)
        self._fill_lookup_prop(properties)
        return self._prettify(top)

    def write_xml(self, xml, output_file=None):
        """Outputs the XML content into a file."""
        gen_filename = 'testrun_{}-{:%Y%m%d%H%M%S}.xml'.format(
            self.testrun_id, datetime.datetime.now())
        utils.write_xml(xml, output_loc=output_file, filename=gen_filename)



# encoding: utf-8
# pylint: disable=missing-docstring,redefined-outer-name,no-self-use

import os
import io
import shutil
import pytest

from mock import patch

from tests import conf

from dump2polarion import dbtools
from dump2polarion import dumper_cli
from dump2polarion.exceptions import Dump2PolarionException
from dump2polarion.transform import only_passed_and_wait


class TestDumperCLIUnits(object):
    def test_get_args(self):
        args = dumper_cli.get_args(['-i', 'dummy', '-t', 'testrun_id'])
        assert args.input_file == 'dummy'
        assert args.output_file is None
        assert args.testrun_id == 'testrun_id'
        assert args.config_file is None
        assert args.no_submit is False
        assert args.user is None
        assert args.password is None
        assert args.force is False
        assert args.log_level is None

    def test_testrun_id_match(self):
        args = dumper_cli.get_args(['-i', 'dummy', '-t', '5_8_0_17'])
        found = dumper_cli.get_testrun_id(args, '5_8_0_17')
        assert found == '5_8_0_17'

    def test_testrun_id_nomatch(self):
        args = dumper_cli.get_args(['-i', 'dummy', '-t', '5_8_0_17'])
        with pytest.raises(Dump2PolarionException) as excinfo:
            dumper_cli.get_testrun_id(args, '5_8_0_18')
        assert "found in exported data doesn't match" in str(excinfo.value)

    def test_testrun_id_force(self):
        args = dumper_cli.get_args(['-i', 'dummy', '-t', '5_8_0_18', '--force'])
        found = dumper_cli.get_testrun_id(args, '5_8_0_17')
        assert found == '5_8_0_18'

    def test_testrun_id_missing(self):
        args = dumper_cli.get_args(['-i', 'dummy'])
        with pytest.raises(Dump2PolarionException) as excinfo:
            dumper_cli.get_testrun_id(args, None)
        assert 'The testrun id was not specified' in str(excinfo.value)

    def test_submit_if_ready_noxml(self, config_prop):
        args = dumper_cli.get_args(['-i', 'submit.txt'])
        submit_args = dumper_cli.get_submit_args(args)
        with patch('dump2polarion.submit_and_verify', return_value=True):
            retval = dumper_cli.submit_if_ready(args, submit_args, config_prop)
        assert retval is None

    def test_submit_if_ready_nosubmit(self, tmpdir, config_prop):
        xml_content = '<testcases foo=bar>'
        xml_file = tmpdir.join('submit_nosubmit.xml')
        xml_file.write(xml_content)
        args = dumper_cli.get_args(['-i', str(xml_file), '--no-submit'])
        submit_args = dumper_cli.get_submit_args(args)
        with patch('dump2polarion.submit_and_verify', return_value=True):
            retval = dumper_cli.submit_if_ready(args, submit_args, config_prop)
        assert retval == 0

    def test_submit_if_ready_failed(self, tmpdir, config_prop):
        xml_content = '<testcases foo=bar>'
        xml_file = tmpdir.join('submit_failed.xml')
        xml_file.write(xml_content)
        args = dumper_cli.get_args(['-i', str(xml_file)])
        submit_args = dumper_cli.get_submit_args(args)
        with patch('dump2polarion.submit_and_verify', return_value=False):
            retval = dumper_cli.submit_if_ready(args, submit_args, config_prop)
        assert retval == 2

    @pytest.mark.parametrize('tag', ('testsuites', 'testcases'))
    def test_submit_if_ready_ok(self, tmpdir, config_prop, tag):
        xml_content = '<{} foo=bar>'.format(tag)
        xml_file = tmpdir.join('submit_ready.xml')
        xml_file.write(xml_content)
        args = dumper_cli.get_args(['-i', str(xml_file)])
        submit_args = dumper_cli.get_submit_args(args)
        with patch('dump2polarion.submit_and_verify', return_value=True):
            retval = dumper_cli.submit_if_ready(args, submit_args, config_prop)
        assert retval == 0


E2E_DATA = [
    # default transform
    ('junit-report.xml', 'junit_transform.xml', ['-t', '5_8_0_17'], None),
    ('workitems_ids.csv', 'complete_transform.xml', [], None),
    ('workitems_ids.sqlite3', 'complete_transform.xml', [], None),
    ('ostriz.json', 'ostriz_transform.xml', [], None),
    # no transform
    ('junit-report.xml', 'junit_notransform.xml', ['-t', '5_8_0_17'], lambda arg: arg),
    ('workitems_ids.csv', 'complete_notransform.xml', [], lambda arg: arg),
    ('workitems_ids.sqlite3', 'complete_notransform.xml', [], lambda arg: arg),
    ('ostriz.json', 'ostriz_notransform.xml', [], lambda arg: arg),
    # only passed and wait
    ('junit-report.xml', 'junit_passed_wait_transform.xml', ['-t', '5_8_0_17'],
     only_passed_and_wait),
    ('workitems_ids.csv', 'complete_passed_wait_transform.xml', [], only_passed_and_wait),
    ('workitems_ids.sqlite3', 'complete_passed_wait_transform.xml', [], only_passed_and_wait),
    ('ostriz.json', 'ostriz_passed_wait_transform.xml', [], only_passed_and_wait),
]


class TestDumperCLIE2E(object):
    @pytest.mark.parametrize('data', E2E_DATA, ids=[d[0] for d in E2E_DATA])
    @pytest.mark.parametrize('submit', (True, False), ids=('submit', 'nosubmit'))
    # pylint: disable=too-many-locals
    def test_main_formats(self, tmpdir, config_e2e, data, submit):
        input_name, golden_output, extra_args, transform_func = data
        # copy the sqlite db so the records are not marked as exported
        if 'sqlite3' in input_name:
            orig_db_file = os.path.join(conf.DATA_PATH, input_name)
            db_file = os.path.join(str(tmpdir), 'workitems_copy.sqlite3')
            shutil.copy(orig_db_file, db_file)
            input_file = db_file
        else:
            input_file = os.path.join(conf.DATA_PATH, input_name)
        output_file = tmpdir.join('out.xml')
        args = ['-i', input_file, '-o', str(output_file), '-c', config_e2e]
        args.extend(extra_args)
        if not submit:
            args.append('-n')

        with patch('dump2polarion.submit_and_verify', return_value=True),\
                patch('dump2polarion.dumper_cli.utils.init_log'):
            retval = dumper_cli.main(args, transform_func=transform_func)
        assert retval == 0

        with io.open(os.path.join(conf.DATA_PATH, golden_output), encoding='utf-8') as golden_xml:
            parsed = golden_xml.read()
        with io.open(str(output_file), encoding='utf-8') as out_xml:
            produced = out_xml.read()
        assert produced == parsed

    def test_main_missing_testrun(self, tmpdir, config_e2e, captured_log):
        input_file = os.path.join(conf.DATA_PATH, 'junit-report.xml')
        output_file = tmpdir.join('out.xml')
        args = ['-i', input_file, '-o', str(output_file), '-c', config_e2e, '-n']

        with patch('dump2polarion.submit_and_verify', return_value=True):
            retval = dumper_cli.main(args)
        assert retval == 1
        assert 'The testrun id was not specified' in captured_log.getvalue()

        with pytest.raises(IOError) as excinfo:
            open(str(output_file))
        assert 'No such file or directory' in str(excinfo.value)

    def test_main_submit_ready(self, config_e2e):
        input_file = os.path.join(conf.DATA_PATH, 'complete_transform.xml')
        args = ['-i', input_file, '-c', config_e2e]

        with patch('dump2polarion.submit_and_verify', return_value=True), \
                patch('dump2polarion.dumper_cli.utils.init_log'):
            retval = dumper_cli.main(args)
        assert retval == 0

    def test_main_noreport(self, config_e2e):
        input_file = os.path.join(conf.DATA_PATH, 'noreport.csv')
        args = ['-i', input_file, '-c', config_e2e]

        with patch('dump2polarion.submit_and_verify', return_value=True), \
                patch('dump2polarion.dumper_cli.utils.init_log'):
            retval = dumper_cli.main(args)
        assert retval == 0

    def test_main_noresults(self, config_e2e, captured_log):
        input_file = os.path.join(conf.DATA_PATH, 'noresults.csv')
        args = ['-i', input_file, '-c', config_e2e]

        with patch('dump2polarion.submit_and_verify', return_value=True):
            retval = dumper_cli.main(args)
        assert retval == 1
        assert 'No results read from' in captured_log.getvalue()

    def test_main_unconfigured(self, captured_log):
        input_file = os.path.join(conf.DATA_PATH, 'noreport.csv')
        args = ['-i', input_file]

        with patch('dump2polarion.submit_and_verify', return_value=True):
            retval = dumper_cli.main(args)
        assert retval == 1
        assert 'Failed to find following keys in config file' in captured_log.getvalue()

    def test_main_noconfig(self, captured_log):
        input_file = os.path.join(conf.DATA_PATH, 'noreport.csv')
        args = ['-i', input_file, '-c', 'nonexistent']

        with patch('dump2polarion.submit_and_verify', return_value=True):
            retval = dumper_cli.main(args)
        assert retval == 1
        assert 'Cannot open config file' in captured_log.getvalue()

    def test_main_submit_failed(self, tmpdir, config_e2e):
        input_file = os.path.join(conf.DATA_PATH, 'workitems_ids.csv')
        output_file = tmpdir.join('out.xml')
        args = ['-i', input_file, '-o', str(output_file), '-c', config_e2e]

        with patch('dump2polarion.submit_and_verify', return_value=False), \
                patch('dump2polarion.dumper_cli.utils.init_log'):
            retval = dumper_cli.main(args)
        assert retval == 2

        golden_output = 'complete_transform.xml'
        with io.open(os.path.join(conf.DATA_PATH, golden_output), encoding='utf-8') as golden_xml:
            parsed = golden_xml.read()
        with io.open(str(output_file), encoding='utf-8') as out_xml:
            produced = out_xml.read()
        assert produced == parsed

    # pylint: disable=too-many-locals,protected-access
    def test_main_submit_db(self, tmpdir, config_e2e):
        orig_db_file = os.path.join(conf.DATA_PATH, 'workitems_ids.sqlite3')
        db_file = os.path.join(str(tmpdir), 'workitems_copy.sqlite3')
        shutil.copy(orig_db_file, db_file)

        output_file = tmpdir.join('out.xml')
        args = ['-i', db_file, '-o', str(output_file), '-c', config_e2e]

        with patch('dump2polarion.submit_and_verify', return_value=True), \
                patch('dump2polarion.dumper_cli.utils.init_log'):
            retval = dumper_cli.main(args)
        assert retval == 0

        golden_output = 'complete_transform.xml'
        with io.open(os.path.join(conf.DATA_PATH, golden_output), encoding='utf-8') as golden_xml:
            parsed = golden_xml.read()
        with io.open(str(output_file), encoding='utf-8') as out_xml:
            produced = out_xml.read()
        assert produced == parsed

        conn = dbtools._open_sqlite(db_file)
        cur = conn.cursor()
        select = "SELECT count(*) FROM testcases WHERE exported == 'yes'"
        cur.execute(select)
        num = cur.fetchone()
        conn.close()
        assert num[0] == 13



# -*- coding: utf-8 -*-
# pylint: disable=missing-docstring,redefined-outer-name,no-self-use,protected-access,invalid-name

from __future__ import unicode_literals

import pytest

from dump2polarion import transform

# format:
# original record, expected record, parameter id

NOT_PASSED = [
    (
        {'title': 'test1', 'verdict': 'failed'},
        None,
        'failed_removed',
    ),
    (
        {'title': 'test1', 'verdict': 'skipped'},
        None,
        'skipped_nobz',
    ),
    (
        {'title': 'test1', 'verdict': 'skipped', 'comment': 'BZ123'},
        {'title': 'test1', 'verdict': 'skipped', 'comment': 'BZ123'},
        'skipped_bz',
    ),
    (
        {'title': 'test1', 'verdict': 'skipped', 'comment': 'BZ 123'},
        {'title': 'test1', 'verdict': 'skipped', 'comment': 'BZ 123'},
        'skipped_bz',
    ),
    (
        {'title': 'test1', 'verdict': 'skipped', 'comment': 'SKIPME: foo'},
        {'title': 'test1', 'verdict': 'skipped', 'comment': 'foo'},
        'skipped_skipme',
    ),
    (
        {'title': 'test1', 'verdict': 'failed', 'comment': 'FAILME: foo'},
        {'title': 'test1', 'verdict': 'failed', 'comment': 'foo'},
        'failed_failme',
    ),
    (
        {'title': 'test1', 'verdict': 'wait'},
        {'title': 'test1', 'verdict': 'wait'},
        'waiting',
    ),
]

RHCF3_ONLY = [
    (
        {'classname': 'cfme.tests.rest.TestRESTAPI', 'title': 'test1', 'verdict': 'passed',
         'comment': 'comment', 'file': 'cfme/tests/rest.py'},
        {'title': 'TestRESTAPI.test1', 'verdict': 'passed', 'comment': 'comment',
         'file': 'cfme/tests/rest.py'},
        'append_class',
    ),
    (
        {'classname': 'TestRESTAPI', 'title': 'test1', 'verdict': 'passed',
         'comment': 'comment', 'file': 'cfme/tests/rest.py'},
        {'title': 'test1', 'verdict': 'passed', 'comment': 'comment',
         'file': 'cfme/tests/rest.py'},
        'no_append_class',
    ),
    (
        {'classname': 'cfme.tests.rest', 'title': 'test1', 'verdict': 'passed',
         'comment': 'comment', 'file': 'cfme/tests/rest.py'},
        {'title': 'test1', 'verdict': 'passed', 'comment': 'comment',
         'file': 'cfme/tests/rest.py'},
        'no_append_class',
    ),
    (
        {'title': 'test1', 'verdict': 'passed',
         'source': 'jenkins', 'job_name': 'downstream', 'run': '123'},
        {'title': 'test1', 'verdict': 'passed', 'comment': 'Source: jenkins/downstream/123',
         'source': 'jenkins', 'job_name': 'downstream', 'run': '123'},
        'passed_comment',
    ),
]

CMP_ONLY = [
    (
        {'classname': 'test1', 'title': 'test1', 'verdict': 'passed', 'test_id': 'CMP-9985'},
        {'title': 'test1', 'verdict': 'passed', 'test_id': 'CMP-9985', 'id': 'CMP-9985'},
        'add_id',
    ),
]

RHCF3_DATA = RHCF3_ONLY + NOT_PASSED
CMP_DATA = CMP_ONLY + NOT_PASSED


class TestTransform(object):
    @pytest.fixture(scope='class')
    def config_rhcf3(self):
        return {'xunit_import_properties': {'polarion-project-id': 'RHCF3'}}

    @pytest.fixture(scope='class')
    def config_cmp(self):
        return {'xunit_import_properties': {'polarion-project-id': 'CMP'}}

    @pytest.mark.parametrize('data', RHCF3_DATA, ids=[d[2] for d in RHCF3_DATA])
    def test_transform_rhcf3(self, config_rhcf3, data):
        tfunc = transform.get_results_transform(config_rhcf3)
        result = tfunc(data[0])
        assert result == data[1]

    @pytest.mark.parametrize('data', CMP_DATA, ids=[d[2] for d in CMP_DATA])
    def test_transform_cmp(self, config_cmp, data):
        tfunc = transform.get_results_transform(config_cmp)
        result = tfunc(data[0])
        assert result == data[1]



# encoding: utf-8
# pylint: disable=missing-docstring,redefined-outer-name,no-self-use,too-few-public-methods

from __future__ import unicode_literals

import pytest

from dump2polarion.exceptions import Dump2PolarionException
from dump2polarion.importer import _get_importer

from dump2polarion.ostriztools import import_ostriz
from dump2polarion.junittools import import_junit
from dump2polarion.csvtools import import_csv
from dump2polarion.dbtools import import_sqlite, SQLITE_EXT


class TestImporterFormats(object):
    def test_ostriz_local(self):
        importer = _get_importer('test_ostriz_file.json')
        assert importer == import_ostriz

    def test_ostriz_remote(self):
        importer = _get_importer(
            'http://trackerbot/ostriz/jenkins/jenkins/downstream-58z-extcloud-tests-master')
        assert importer == import_ostriz

    def test_junit(self):
        importer = _get_importer('test_junit.xml')
        assert importer == import_junit

    def test_csv(self):
        importer = _get_importer('workitems.csv')
        assert importer == import_csv

    @pytest.mark.parametrize('ext', SQLITE_EXT)
    def test_db(self, ext):
        importer = _get_importer('workitems{}'.format(ext))
        assert importer == import_sqlite

    def test_invalid(self):
        with pytest.raises(Dump2PolarionException) as excinfo:
            _get_importer('workitems.txt')
        assert 'Cannot recognize type of input data, add file extension' in str(excinfo.value)



# -*- coding: utf-8 -*-
# pylint: disable=logging-format-interpolation
"""
Helper functions for handling data in pytest junit format.
"""

from __future__ import absolute_import, unicode_literals

import os

from collections import OrderedDict

from xml.etree import ElementTree

from dump2polarion import exporter
from dump2polarion.exceptions import Dump2PolarionException


# pylint: disable=unused-argument
def import_junit(junit_file, **kwargs):
    """Reads the content of the junit-results file produced by pytest and returns imported data."""
    try:
        tree = ElementTree.parse(os.path.expanduser(junit_file))
    except Exception as err:
        raise Dump2PolarionException("Failed to parse XML file '{}': {}".format(junit_file, err))

    xml_root = tree.getroot()

    results = []
    for test_data in xml_root:
        if test_data.tag != 'testcase':
            continue

        verdict = None
        verdict_found = False
        comment = ''
        properties = {}
        for element in test_data:
            if not verdict_found:
                if element.tag == 'error':
                    verdict = 'failed'
                    comment = element.get('message')
                    # continue to see if there's more telling verdict for this record
                elif element.tag == 'failure':
                    verdict = 'failed'
                    comment = element.get('message')
                    verdict_found = True
                elif element.tag == 'skipped':
                    verdict = 'skipped'
                    comment = element.get('message')
                    verdict_found = True
            if element.tag == 'properties':
                for prop in element:
                    properties[prop.get('name')] = prop.get('value')
        if not verdict:
            verdict = 'passed'

        title = test_data.get('name')
        classname = test_data.get('classname')
        time = test_data.get('time', 0)
        filepath = test_data.get('file')

        data = [
            ('title', title),
            ('classname', classname),
            ('verdict', verdict),
            ('comment', comment),
            ('time', time),
            ('file', filepath),
        ]
        for key in properties:
            data.append((key, properties[key]))

        results.append(OrderedDict(data))

    return exporter.ImportedData(results=results, testrun=None)



# encoding: utf-8
# pylint: disable=missing-docstring,redefined-outer-name,no-self-use,protected-access

from __future__ import unicode_literals

import os

try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

import pytest
from tests import conf

from dump2polarion import csvtools
from dump2polarion.exceptions import Dump2PolarionException


class TestCSVFileldNames(object):
    def test_fieldnames_exported(self):
        csv_file = os.path.join(conf.DATA_PATH, 'workitems_ids.csv')
        open_args = []
        open_kwargs = {}
        try:
            # pylint: disable=pointless-statement
            unicode
            open_args.append('rb')
        except NameError:
            open_kwargs['encoding'] = 'utf-8'
        with open(csv_file, *open_args, **open_kwargs) as input_file:
            reader = csvtools._get_csv_reader(input_file)
            fieldnames = csvtools._get_csv_fieldnames(reader)
        assert fieldnames == [
            'id',
            'title',
            'testcaseid',
            'caseimportance',
            'verdict',
            'comment',
            'stdout',
            'stderr',
            'exported',
            'time'
        ]

    def test_fieldnames_unanotated(self):
        csv_content = str(',,ID,Title,Test Case I D,Caseimportance')
        input_file = StringIO(csv_content)
        reader = csvtools._get_csv_reader(input_file)
        fieldnames = csvtools._get_csv_fieldnames(reader)
        input_file.close()
        assert fieldnames == [
            'field1',
            'field2',
            'id',
            'title',
            'testcaseid',
            'caseimportance'
        ]

    def test_fieldnames_trailing(self):
        csv_content = str('ID,Title,Test Case I D,Caseimportance,,,')
        input_file = StringIO(csv_content)
        reader = csvtools._get_csv_reader(input_file)
        fieldnames = csvtools._get_csv_fieldnames(reader)
        input_file.close()
        assert fieldnames == [
            'id',
            'title',
            'testcaseid',
            'caseimportance'
        ]

    def test_fieldnames_missing_id(self):
        csv_content = str('Title,Test Case I D,Caseimportance,,,')
        input_file = StringIO(csv_content)
        reader = csvtools._get_csv_reader(input_file)
        fieldnames = csvtools._get_csv_fieldnames(reader)
        input_file.close()
        assert fieldnames is None


class TestCSVTestrunId(object):
    def test_testrun_id_exported(self):
        csv_file = os.path.join(conf.DATA_PATH, 'workitems_ids.csv')
        open_args = []
        open_kwargs = {}
        try:
            # pylint: disable=pointless-statement
            unicode
            open_args.append('rb')
        except NameError:
            open_kwargs['encoding'] = 'utf-8'
        with open(csv_file, *open_args, **open_kwargs) as input_file:
            reader = csvtools._get_csv_reader(input_file)
            testrun_id = csvtools._get_testrun_from_csv(input_file, reader)
        assert testrun_id == '5_8_0_17'

    def test_testrun_id_line(self):
        csv_content = str('Query,"(assignee.id:$[user.id] AND NOT status:inactive AND '
                          '(TEST_RECORDS:(""RHCF3/5_8_0_17"", @null))) '
                          'AND project.id:RHCF3",,,,,,,')
        input_file = StringIO(csv_content)
        reader = csvtools._get_csv_reader(input_file)
        testrun_id = csvtools._get_testrun_from_csv(input_file, reader)
        assert testrun_id == '5_8_0_17'

    def test_testrun_id_far(self):
        csv_content = str("""
            ,ID,Title,Test Case I D,,,,
            ,(TEST_RECORDS:(""RHCF3/5_8_0_17"", @null)),,,,,""")
        input_file = StringIO(csv_content)
        reader = csvtools._get_csv_reader(input_file)
        testrun_id = csvtools._get_testrun_from_csv(input_file, reader)
        assert not testrun_id


class TestCSVImport(object):
    def test_import_orig_data(self):
        csv_file = os.path.join(conf.DATA_PATH, 'workitems_ids.csv')
        data = csvtools.get_imported_data(csv_file)
        assert hasattr(data, 'results')
        assert len(data.results) == 15
        assert 'id' in data.results[0]
        assert hasattr(data, 'testrun')
        assert data.testrun == '5_8_0_17'

    def test_import_no_results(self, tmpdir):
        csv_content = str('ID,Title,Test Case I D,Caseimportance,,,')
        csv_file = tmpdir.join('no_results.csv')
        csv_file.write(csv_content)
        csv_file_path = str(csv_file)
        with pytest.raises(Dump2PolarionException) as excinfo:
            csvtools.get_imported_data(csv_file_path)
        assert 'No results read from CSV file' in str(excinfo.value)

    def test_import_and_check_verdict(self, tmpdir):
        csv_content = str("""
            ,ID,Title,Test Case I D,,,,
            ,RH123,,,,,,""")
        csv_file = tmpdir.join('invalid_fieldnames.csv')
        csv_file.write(csv_content)
        csv_file_path = str(csv_file)
        with pytest.raises(Dump2PolarionException) as excinfo:
            csvtools.import_csv(csv_file_path)
        assert 'missing following columns: Verdict' in str(excinfo.value)



# -*- coding: utf-8 -*-
"""
Functions for validating and transforming results. These are specific for given Polarion project.

If the 'polarion-lookup-method' is set to 'custom', this is the place where you can
set the 'id' of the testcase to desired value.
"""

from __future__ import absolute_import, unicode_literals

import re

from dump2polarion.verdicts import Verdicts


# pylint: disable=inconsistent-return-statements
def only_passed_and_wait(result):
    """Returns PASS and WAIT results only, skips everything else."""
    verdict = result.get('verdict', '').strip().lower()
    if verdict in Verdicts.PASS + Verdicts.WAIT:
        return result


def _insert_source_info(result):
    """Adds info about source of test result if available."""
    comment = result.get('comment')
    # don't change comment if it already exists
    if comment:
        return

    source = result.get('source')
    job_name = result.get('job_name')
    run = result.get('run')
    source_list = [source, job_name, run]
    if not all(source_list):
        return

    source_note = '/'.join(source_list)
    source_note = 'Source: {}'.format(source_note)
    result['comment'] = source_note


# pylint: disable=unused-argument
def get_results_transform_cfme(config):
    """Return result transformation function for CFME."""
    skip_searches = [
        'SKIPME:',
        'Skipping due to these blockers',
        'BZ ?[0-9]+',
        'GH ?#?[0-9]+',
        'GH#ManageIQ',
    ]
    skips = re.compile('(' + ')|('.join(skip_searches) + ')')

    def results_transform(result):
        """Results transform for CFME."""
        verdict = result.get('verdict')
        if not verdict:
            return None

        result = result.copy()

        # make sure that last part of classname is included in "title", e.g.
        # "TestServiceRESTAPI.test_power_parent_service"
        classname = result.get('classname', '')
        if classname:
            filepath = result.get('file', '')
            title = result.get('title')
            if title and '/' in filepath and '.' in classname:
                fname = filepath.split('/')[-1].replace('.py', '')
                last_classname = classname.split('.')[-1]
                # last part of classname is not file name
                if fname != last_classname and last_classname not in title:
                    result['title'] = '{0}.{1}'.format(last_classname, title)
            # we don't need to pass classnames?
            del result['classname']

        # add source of test result if available
        _insert_source_info(result)

        verdict = verdict.strip().lower()
        # we want to submit PASS and WAIT results
        if verdict in Verdicts.PASS + Verdicts.WAIT:
            return result
        comment = result.get('comment')
        # ... and SKIP results where there is a good reason (blocker etc.)
        if verdict in Verdicts.SKIP and comment and skips.search(comment):
            # found reason for skip
            result['comment'] = comment.replace('SKIPME: ', '').replace('SKIPME', '')
            return result
        if verdict in Verdicts.FAIL and comment and 'FAILME' in comment:
            result['comment'] = comment.replace('FAILME: ', '').replace('FAILME', '')
            return result
        # we don't want to report this result if here
        return None

    return results_transform


# pylint: disable=unused-argument
def get_results_transform_cmp(config):
    """Return result transformation function for CFME."""
    skip_searches = [
        'SKIPME:',
        'Skipping due to these blockers',
        'BZ ?[0-9]+',
        'GH ?#?[0-9]+',
        'GH#ManageIQ',
    ]
    skips = re.compile('(' + ')|('.join(skip_searches) + ')')

    def results_transform(result):
        """Results transform for CMP."""
        verdict = result.get('verdict')
        if not verdict:
            return None

        result = result.copy()

        classname = result.get('classname', '')
        if classname:
            # we don't need to pass classnames?
            del result['classname']

        # if the "test_id" property is present, use it as testcase ID
        test_id = result.get('test_id', '')
        if test_id:
            result['id'] = test_id

        verdict = verdict.strip().lower()
        # we want to submit PASS and WAIT results
        if verdict in Verdicts.PASS + Verdicts.WAIT:
            return result
        comment = result.get('comment')
        # ... and SKIP results where there is a good reason (blocker etc.)
        if verdict in Verdicts.SKIP and comment and skips.search(comment):
            # found reason for skip
            result['comment'] = comment.replace('SKIPME: ', '').replace('SKIPME', '')
            return result
        if verdict in Verdicts.FAIL and comment and 'FAILME' in comment:
            result['comment'] = comment.replace('FAILME: ', '').replace('FAILME', '')
            return result
        # we don't want to report this result if here
        return None

    return results_transform


PROJECT_MAPPING = {
    'RHCF3': get_results_transform_cfme,
    'CMP': get_results_transform_cmp,
}


# pylint: disable=inconsistent-return-statements
def get_results_transform(config):
    """Returns results transformation function.

    The transformation function is returned by calling corresponding "getter" function.

    This allows customizations of results data according to requirements
    of the specific project.

    When no results data are returned, this result will be ignored
    and will not be written to the resulting XML.
    """

    project = config['xunit_import_properties']['polarion-project-id']
    if project in PROJECT_MAPPING:
        return PROJECT_MAPPING[project](config)
